# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from augeas import Augeas
from Cheetah.Template import Template
from subprocess import call, Popen, PIPE
from os.path import isfile, abspath, join, exists

from pykolab import utils
from pykolab.translate import _

import pykolab
import sys

conf = pykolab.getConf()

# Commands with service suffix usage and argument ordering "format".

commands_and_options = {
    '/bin/systemctl'        : (True,  ('cmd', 'operation', 'service')),
    '/sbin/service'         : (False, ('cmd', 'service', 'operation')),
    '/usr/sbin/service'     : (False, ('cmd', 'service', 'operation')),
    '/sbin/chkconfig'       : (False, ('cmd', 'service', 'operation')),
    '/usr/sbin/update-rc.d' : (False, ('cmd', 'service', 'operation')),
    }

# Configuration options corresponding to false and true values.

configuration_options = {
    '/bin/systemctl'        : ('disable', 'enable'),
    '/sbin/chkconfig'       : ('off', 'on'),
    '/usr/sbin/update-rc.d' : ('disable', 'defaults'),
    }

configuration_commands = configuration_options.keys()
control_commands = '/bin/systemctl', '/sbin/service', '/usr/sbin/service'

configuration_command_details = None
control_command_details = None

def get_command_options(commands):
    """
        Check for the presence of the given commands, returning the first one
        found along with option details.
    """
    for name in commands:
        if isfile(name):
            return name, commands_and_options[name]
    return None, None

def get_control_command_details():
    """
        Obtain and cache the control command details.
    """
    global control_command_details
    if control_command_details is None:
        control_command_details = get_command_options(control_commands)
    return control_command_details

def get_configuration_command_details():
    """
        Obtain and cache the configuration command details.
    """
    global configuration_command_details
    if configuration_command_details is None:
        configuration_command_details = get_command_options(configuration_commands)
    return configuration_command_details

def apply_command(name, suffix, format, service_name, operation):
    """
        Call the given command, adding the applicable service name suffix to the
        service name argument, using the "format" to determine the argument
        ordering, and employing the service name and operation specified.
    """
    cmd = []
    for arg in format:
        if arg == "cmd":
            cmd.append(name)
        elif arg == "operation":
            cmd.append(operation)
        elif arg == "service":
            cmd.append('%s%s' % (service_name, suffix))
    call(cmd, stdout=PIPE, stderr=PIPE)

def control_service(service_name, operation, suffix=".service"):
    """
        Perform a service control operation on the service with the given name.
        Where a service command employs a suffix on the service name, any
        provided suffix will be added.
    """
    name, (use_suffix, format) = get_control_command_details()
    if not name:
        return False
    apply_command(name, use_suffix and suffix or "", format, service_name, operation)
    return True

def configure_service(service_name, operation, suffix=".service"):
    """
        Perform a service configuration operation on the service with the given
        name. Where a service command employs a suffix on the service name, any
        provided suffix will be added.
    """
    name, (use_suffix, format) = get_configuration_command_details()
    if not name:
        return False
    options = configuration_options[name]
    operation = options[operation and 1 or 0]
    apply_command(name, use_suffix and suffix or "", format, service_name, operation)
    return True

def is_debian():
    """
        Special test to deal with inconsistencies in package naming and file
        locations.
    """
    return exists("/etc/debian_version")

def find_file(name, paths):
    for path in paths:
        full_path = join(path, name)
        if isfile(full_path):
            return full_path
    return None

def file_contains_data(filename, data, exact=False):
    if not exists(filename):
        return False
    f = open(filename)
    try:
        if exact:
            return f.read() == data
        else:
            return f.read().find(data) != -1
    finally:
        f.close()

# Message and input handling.

# These functions wrap the existing utils functions and provide for debconf
# usage instead of using the plain console. The principal additions to the
# existing functions involve message keys (to template definitions accessed by
# debconf, provided by Debian packages) and the messages that are displayed
# before each question.

debconf = None
db = None
db_passwords = set()

def use_debconf():
    return is_debian() and conf.use_system_config

def start_interaction(message_key):
    global db, debconf
    if use_debconf():
        try:
            import debconf
        except ImportError:
            return
        debconf.runFrontEnd()
        db = debconf.Debconf()
        db.settitle(message_key)

def ask_question(message_key, message, question=None, default="", password=False, confirm=False):
    """
        Use the given message key to retrieve a question from any
        system-supplied configuration system, otherwise using the given message
        and question to prompt the user for an answer. Where no question is
        actually given, this function just displays a message.
    """
    if db is not None:
        if password:
            db_passwords.add(message_key)
        if default:
            db.set(message_key, default)
            db.fset(message_key, "seen", False)
        try:
            db.input("high", message_key)
            db.go()
            return db.getString(message_key)
        except debconf.DebconfError:
            return None
    else:
        print >> sys.stderr, utils.multiline_message(message)
        if question:
            return utils.ask_question(question, default, password, confirm)
        else:
            return None

def stop_interaction():
    if db is not None:

        # Erase password information from debconf as suggested by...
        # http://www.debian.org/doc/manuals/developers-reference/best-pkging-practices.html#s6.5.3

        for message_key in db_passwords:
            db.set(message_key, "")
        db.stop()

# PHP-related functions.

php_paths = ['/etc', '/etc/php5/apache2']

def get_php_path(name):
    return find_file(name, php_paths)

def get_php_ini_file():
    if conf.php_ini_path:
        if not isfile(conf.php_ini_path):
            return None
        else:
            return conf.php_ini_path
    else:
        return get_php_path("php.ini")

# Template-related functions.

template_paths = ['/etc/kolab/templates', '/usr/share/kolab/templates']

def get_template_path(name):
    """
        Return the full path to the template with the given name.
    """
    relative_path = abspath(join(__file__, '..', '..', '..', 'share', 'templates', name))
    return find_file(name, template_paths + [relative_path])

def instantiate_template(template_file, output_file, search_list, check_only=False):
    """
        Instantiate the template from the given file, producing the indicated
        output file. The search list provides a collection of definition
        mappings for substitutions.

        When only checking template instantiation, the output file is not
        produced, but instead an indication of whether the output file exists
        and matches is returned.
    """
    fp = open(template_file, 'r')
    template_definition = fp.read()
    fp.close()

    t = Template(template_definition, searchList=search_list)
    data = str(t)
    if check_only:
        return file_contains_data(output_file, data, exact=True)
    else:
        fp = open(output_file, 'w')
        fp.write(data)
        fp.close()
        return False

# Defaults-related functions.

def get_system_timezone():
    if not exists("/etc/timezone"):
        return None
    f = open("/etc/timezone")
    try:
        return f.read().strip()
    finally:
        f.close()

def set_service_default(name, setting_name, setting_value):
    if isfile(join('/etc/default', name)):
        myaugeas = Augeas()
        setting = join('/files/etc/default/%s' % name, setting_name)

        if myaugeas.get(setting) != setting_value:
            myaugeas.set(setting, setting_value)
            myaugeas.save()

        myaugeas.close()

# Database-related functions.

def grep_for_expression(preceding, expr):
    p = Popen(['/bin/grep', expr], stdin=preceding.stdout, stdout=PIPE, stderr=PIPE)
    return p.wait() == 0

def get_mysql_defaults():
    if is_debian(): 
        return '/etc/mysql/debian.cnf'
    else:
        return '/tmp/kolab-setup-my.cnf'

def need_mysql_root():
    return not is_debian()

def have_mysql_database(config, name):
    p = Popen(['mysql'] + (config and ['--defaults-file=%s' % config] or []) + [
        '-e', 'show databases',
        '--batch', '--skip-column-names'],
        stdout=PIPE,
        stderr=PIPE
        )
    return grep_for_expression(p, name)

def have_mysql_user(config, name):
    return have_mysql_data(config, "mysql",
        "select User from user where User = '%s' and Host = 'localhost'" % name,
        name)

def have_mysql_data(config, database, query, expected):
    p = Popen(['mysql'] + (config and ['--defaults-file=%s' % config] or []) + [
        '-D', database,
        '-e', query,
        '--batch', '--skip-column-names'],
        stdout=PIPE,
        stderr=PIPE
        )
    return grep_for_expression(p, expected)

def have_mysql_process():
    pidfile = '/var/run/mysqld/mysqld.pid'
    return have_process_with_file(pidfile)

def get_mysql_defaults():
    for path in '/etc/mysql/debian.cnf', '/etc/mysql/my.cnf', '/etc/my.cnf':
        if exists(path):
            return path
    return None

def make_mysql_defaults_file(defaults_file):
    utils.multiline_message(
            _("Please supply the MySQL root password")
        )

    mysql_root_password = utils.ask_question(
            _("MySQL root password"),
            password=True
        )

    data = """
[mysql]
user=root
password='%s'
""" % mysql_root_password

    fp = open(defaults_file, 'w')
    try:
        os.chmod(defaults_file, 0600)
        fp.write(data)
    finally:
        fp.close()

# LDAP-related functions.

def have_slapd_process(instance):
    pidfile = '/var/run/dirsrv/slapd-%s.pid' % instance
    return have_process_with_file(pidfile)

# Process-related functions.

def have_process_with_file(pidfile):
    if not exists(pidfile):
        return False
    f = open(pidfile)
    try:
        pid = f.read().strip()
        return have_process(pid)
    finally:
        f.close()

def have_process(pid):
    return call(['/bin/ps', '-p', pid]) == 0
